/*
 * Compiz Tiles Plugin
 *
 * animatedrects.h
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _TILES_ANIMATEDRECTS_H
#define _TILES_ANIMATEDRECTS_H

#include <boost/shared_ptr.hpp>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <core/rect.h>

class AnimatableRectInterface
{
    public:

	typedef boost::shared_ptr <AnimatableRectInterface> Ptr;

	virtual ~AnimatableRectInterface () {}

	enum class Type
	{
	    In = 0,
	    Out,
	    Complete
	};

    	virtual void setFromRectFunc (const boost::function<CompRect ()> &) = 0;
	virtual void setToRectFunc (const boost::function<CompRect ()> &) = 0;
    	virtual bool setDirection (Type) = 0;
	virtual bool progressAnimation (int ms, bool force) = 0;

	virtual CompRect tweenedGeometry (const CompRect &rect) = 0;
	virtual unsigned int getOpacity () = 0;
};

class PrivateLinearAnimatedRect;

class LinearAnimatedRect :
    public AnimatableRectInterface
{
    public:

	LinearAnimatedRect ();

	void setFromRectFunc (const boost::function <CompRect ()> &);
	void setToRectFunc (const boost::function<CompRect ()> &);
	bool setDirection (AnimatableRectInterface::Type);

	bool progressAnimation (int ms, bool force);
	CompRect tweenedGeometry (const CompRect &rect);

	CompRect tweenedGeometry ();
	unsigned int getOpacity ();

    private:

	PrivateLinearAnimatedRect *priv;
};

CompRect
scaleChild (const CompRect &child, const CompRect &parent, const CompRect &newParent);

#endif
