#include <gtest/gtest.h>
#include <gmock/gmock.h>

#include "animatedrects.h"

class TilesAnimatedRectsTest :
    public ::testing::Test
{
};

TEST(TilesAnimatedRectsTest, TestNoSetRectGetters)
{
    LinearAnimatedRect lar;

    CompRect r (0, 0, 100, 100);

    EXPECT_EQ (r, lar.tweenedGeometry (r));
}

TEST(TilesAnimatedRectsTest, TestProgressInTweenedGeometryTo)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 400, 400);

    lar.setToRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);
	CompRect g = lar.tweenedGeometry (r);
	CompRect e = r;

	p += 1 / 300.0;

	e.setWidth (r.width () + (exp.width () - r.width ()) * p);
	e.setHeight (r.height () + (exp.height () - r.height ()) * p);

	EXPECT_EQ (g, e);
    }
}

TEST(TilesAnimatedRectsTest, TestProgressInTweenedGeometryFrom)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 50, 50);

    lar.setFromRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);
	CompRect g = lar.tweenedGeometry (r);
	CompRect e = r;

	p += 1 / 300.0;

	e.setWidth (exp.width () + (r.width () - exp.width ()) * p);
	e.setHeight (exp.height () + (r.height () - exp.height ()) * p);

	EXPECT_EQ (g, e);
    }
}

TEST(TilesAnimatedRectsTest, TestProgressInTweenedGeometryToFrom)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 50, 50);
    CompRect texp (0, 0, 100, 100);

    lar.setFromRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setToRectFunc ([&](void) -> CompRect { return texp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);
	CompRect g = lar.tweenedGeometry (r);
	CompRect e = r;

	p += 1 / 300.0;

	e.setWidth (exp.width () + (texp.width () - exp.width ()) * p);
	e.setHeight (exp.height () + (texp.height () - exp.height ()) * p);

	EXPECT_EQ (g, e);
    }
}

TEST(TilesAnimatedRectsTest, TestProgressOutTweenedGeometryTo)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 400, 400);

    lar.setToRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);
	CompRect g = lar.tweenedGeometry (r);
	CompRect e = r;

	p += 1 / 300.0;

	e.setWidth (r.width () + (exp.width () - r.width ()) * p);
	e.setHeight (r.height () + (exp.height () - r.height ()) * p);

	EXPECT_EQ (g, e);
    }
}

TEST(TilesAnimatedRectsTest, TestProgressOutTweenedGeometryFrom)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 50, 50);

    lar.setFromRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);
	CompRect g = lar.tweenedGeometry (r);
	CompRect e = r;

	p += 1 / 300.0;

	e.setWidth (exp.width () + (r.width () - exp.width ()) * p);
	e.setHeight (exp.height () + (r.height () - exp.height ()) * p);

	EXPECT_EQ (g, e);
    }
}

TEST(TilesAnimatedRectsTest, TestProgressOutTweenedGeometryToFrom)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 50, 50);
    CompRect texp (0, 0, 100, 100);

    lar.setFromRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setToRectFunc ([&](void) -> CompRect { return texp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);
	CompRect g = lar.tweenedGeometry (r);
	CompRect e = r;

	p += 1 / 300.0;

	e.setWidth (exp.width () + (texp.width () - exp.width ()) * p);
	e.setHeight (exp.height () + (texp.height () - exp.height ()) * p);

	EXPECT_EQ (g, e);
    }
}

TEST(TilesAnimatedRectsTest, TestProgressOpacity)
{
    LinearAnimatedRect lar;
    CompRect r (0, 0, 100, 100);
    CompRect exp (0, 0, 50, 50);
    CompRect texp (0, 0, 100, 100);

    lar.setFromRectFunc ([&](void) -> CompRect { return exp; } );
    lar.setToRectFunc ([&](void) -> CompRect { return texp; } );
    lar.setDirection (AnimatableRectInterface::Type::In);

    float p = 0.0f;

    for (unsigned int i = 0; i < 300; i++)
    {
	lar.progressAnimation (1, false);

	p += 1 / 300.0;

	short opacity = std::numeric_limits <short>::max () * p;
	EXPECT_EQ (opacity, lar.getOpacity ());
    }
}

std::ostream &
operator<< (std::ostream &os, const CompRect &r)
{
    return os << "CompRect x: " << r.x () <<
		 " y: " << r.y () <<
		 " width: " << r.width () <<
		 " height: " << r.height ();
}

TEST(ScaleRect, TestSimpleRectScale)
{
    CompRect parent (0, 0, 10, 10);
    CompRect child (2, 2, 2, 2);
    CompRect newParent (0, 0, 100, 100);

    CompRect scaledChild = scaleChild (child, parent, newParent);

    EXPECT_EQ (CompRect (20, 20, 20, 20), scaledChild);
}

TEST(ScaleRect, TestTranslatedParentScale)
{
    CompRect parent (0, 0, 10, 10);
    CompRect child (2, 2, 2, 2);
    CompRect newParent (-10, -10, 100, 100);

    CompRect scaledChild = scaleChild (child, parent, newParent);

    EXPECT_EQ (CompRect (10, 10, 20, 20), scaledChild);
}

TEST(ScaleRect, TestComplexScale)
{
    CompRect parent (100, 100, 10, 10);
    CompRect child (102, 102, 2, 2);
    CompRect newParent (-10, -10, 100, 100);

    CompRect scaledChild = scaleChild (child, parent, newParent);

    EXPECT_EQ (CompRect (10, 10, 20, 20), scaledChild);
}

TEST(ScaleRect, TestLargerChild)
{
    CompRect parent (10, 10, 10, 10);
    CompRect child (5, 5, 20, 20);
    CompRect newParent (20, 20, 20, 20);

    CompRect scaledChild (scaleChild (child, parent, newParent));

    EXPECT_EQ (CompRect (10, 10, 40, 40), scaledChild);
}
