/*
 * Compiz Tiles Plugin
 *
 * drawable.h
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _TILES_DND_H
#define _TILES_DND_H

#include <core/rect.h>
#include <boost/function.hpp>

class DroppableInterface
{
    public:

	typedef boost::shared_ptr <DroppableInterface> Ptr;

	virtual ~DroppableInterface () {};
	virtual const CompRect & dropGeometry () = 0;
	virtual void setPotentialDropZone (const CompRect &) = 0;
};

class DraggableInterface
{
    public:

	typedef boost::shared_ptr <DraggableInterface> Ptr;

	virtual ~DraggableInterface () {}
	virtual void applyDrag (const CompPoint &dp) = 0;
	virtual void grab () = 0;
	virtual void ungrab () = 0;
};

class SnapRemovableInterface
{
    public:

	typedef boost::shared_ptr <SnapRemovableInterface> Ptr;
	typedef boost::function <const CompRect & (void)> GetGeometryFunc;

	virtual ~SnapRemovableInterface () {}
	virtual bool shouldSnapOut () = 0;
	virtual void setGetSnapGeometryFunc (const GetGeometryFunc &func) = 0;
};

class DropCommitInterface
{
    public:

	typedef boost::shared_ptr <DropCommitInterface> Ptr;

	virtual ~DropCommitInterface () {}
	virtual void dropCommit () = 0;
};

class PrivateDragAndDrop;

class DragAndDropInterface :
    public DraggableInterface,
    public DroppableInterface,
    public SnapRemovableInterface
{
};

class DragAndDrop :
    public DragAndDropInterface
{
    public:

	DragAndDrop ();

	void setGetSnapGeometryFunc (const SnapRemovableInterface::GetGeometryFunc &func);

	const CompRect &dropGeometry ();
	void setPotentialDropZone (const CompRect &);

	void applyDrag (const CompPoint &dp);
	void grab ();
	void ungrab ();

	bool shouldSnapOut ();

    private:

	PrivateDragAndDrop *priv;
};

#endif
