/*
 * Compiz Tiles Plugin
 *
 * tiles.cpp
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "dnd.h"

class PrivateDragAndDrop
{
    public:

	CompRect mDropZone;
	CompPoint mDragBuffer;
	DragAndDrop::GetGeometryFunc mGetSnapGeometryFunc;
};

DragAndDrop::DragAndDrop () :
    priv (new PrivateDragAndDrop ())
{
}

void
DragAndDrop::setGetSnapGeometryFunc (const GetGeometryFunc &func)
{
    priv->mGetSnapGeometryFunc = func;
}

void
DragAndDrop::grab ()
{
}

void
DragAndDrop::ungrab ()
{
    priv->mDragBuffer = CompPoint (0, 0);
    priv->mDropZone = CompRect ();
}

void
DragAndDrop::setPotentialDropZone (const CompRect &dz)
{
    priv->mDropZone = dz;
}

const CompRect &
DragAndDrop::dropGeometry ()
{
    return priv->mDropZone;
}

const unsigned int max_drag = 20;

bool
DragAndDrop::shouldSnapOut ()
{
    if (fabsf (priv->mDragBuffer.x ()) >= max_drag ||
	fabsf (priv->mDragBuffer.y ()) > max_drag)
	return true;

    return false;
}

void
DragAndDrop::applyDrag (const CompPoint &p)
{
    const CompRect &dropZone = priv->mDropZone;
    const CompRect &snapGeometry (priv->mGetSnapGeometryFunc ());

    /* Left or right ... */
    if (dropZone.height () > dropZone.width ())
    {
	/* Tiling to the right */
	if (dropZone.left () > snapGeometry.left ())
	{
	    priv->mDragBuffer += CompPoint (p.x (), 0);
	    priv->mDragBuffer.setX (std::min (priv->mDragBuffer.x (), 0));
	}
	/* Tiling to the left */
	else
	{
	    priv->mDragBuffer += CompPoint (p.x (), 0);
	    priv->mDragBuffer.setX (std::max (priv->mDragBuffer.x (), 0));
	}
    }
    else
    {
	/* Tiling to the bottom */
	if (dropZone.top () > snapGeometry.top ())
	{
	    priv->mDragBuffer += CompPoint (0, p.y ());
	    priv->mDragBuffer.setY (std::min (priv->mDragBuffer.y (), 0));
	}
	/* Tiling to the top */
	else
	{
	    priv->mDragBuffer += CompPoint (0, p.y ());
	    priv->mDragBuffer.setY (std::max (priv->mDragBuffer.y (), 0));
	}
    }
}
