#include <gtest/gtest.h>
#include <gmock/gmock.h>

#include "dnd.h"

class TilesDndTest:
    public ::testing::Test
{
};

TEST(TilesDndTest, TestSetGetDropZone)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 100, 100));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 100, 100));
}

TEST(TilesDndTest, TestNoSnapOutDragTallDropZoneGreaterThanSnapOutRight)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (1660, 0, 20, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (1660, 0, 20, 1050));

    CompRect g (840, 0, 840, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (-10, 0));

    EXPECT_FALSE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutRight)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (1660, 0, 20, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (1660, 0, 20, 1050));

    CompRect g (840, 0, 840, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (-21, 0));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutRightNoNegImpact)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (1660, 0, 20, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (1660, 0, 20, 1050));

    CompRect g (840, 0, 840, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (-10, 0));
    dnd.applyDrag (CompPoint (11, 0));
    dnd.applyDrag (CompPoint (-21, 0));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestNoSnapOutDragTallDropZoneGreaterThanSnapOutLeft)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 20, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 20, 1050));

    CompRect g (0, 0, 840, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (10, 0));

    EXPECT_FALSE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutLeft)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 20, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 20, 1050));

    CompRect g (0, 0, 840, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (21, 0));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutLeftNoNegImpact)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 20, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 20, 1050));

    CompRect g (0, 0, 840, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (10, 0));
    dnd.applyDrag (CompPoint (-11, 0));
    dnd.applyDrag (CompPoint (21, 0));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestNoSnapOutDragTallDropZoneGreaterThanSnapOutBottom)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 1030, 1680, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 1030, 1680, 1050));

    CompRect g (0, 1030, 1680, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (0, -10));

    EXPECT_FALSE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutBottom)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 1030, 1680, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 1030, 1680, 1050));

    CompRect g (0, 725, 1680, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (0, -21));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutBottomNoNegImpact)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 1030, 1680, 1050));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 1030, 1680, 1050));

    CompRect g (0, 725, 1680, 1050);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (0, -10));
    dnd.applyDrag (CompPoint (0, 11));
    dnd.applyDrag (CompPoint (0, -21));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestNoSnapOutDragTallDropZoneGreaterThanSnapOutTop)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 1680, 20));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 1680, 20));

    CompRect g (0, 0, 1680, 725);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (0, 10));

    EXPECT_FALSE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutTop)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 1680, 20));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 1680, 20));

    CompRect g (0, 0, 1680, 725);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (0, 21));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}

TEST(TilesDndTest, TestSnapOutDragTallDropZoneGreaterThanSnapOutTopNoNegImpact)
{
    DragAndDrop dnd;

    dnd.setPotentialDropZone (CompRect (0, 0, 1680, 20));
    EXPECT_EQ (dnd.dropGeometry (), CompRect (0, 0, 1680, 20));

    CompRect g (0, 0, 1680, 725);

    dnd.setGetSnapGeometryFunc ([&](void) -> const CompRect & { return g; });
    dnd.applyDrag (CompPoint (0, 10));
    dnd.applyDrag (CompPoint (0, -11));
    dnd.applyDrag (CompPoint (0, 21));

    EXPECT_TRUE (dnd.shouldSnapOut ());
}
