/*
 * Compiz Tiles Plugin
 *
 * fbeffect.h
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _TILES_FBEFFECT_H
#define _TILES_FBEFFECT_H

#include <core/size.h>
#include <core/output.h>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <GL/gl.h>
#include <GL/glx.h>

#include <core/core.h>
#include <opengl/opengl.h>

#include "fbmanager.h"

namespace GL
{
    typedef void (*GLUniform1fvProc) (GLuint, GLsizei, GLfloat *);
    extern GLUniform1fvProc glUniform1fvP;
}

class PrivateRebindableGLFramebufferObject;
class PrivateBackingFramebuffer;
class PrivateFramebufferPainter;
class RebindableGLFramebufferObject;

typedef boost::shared_ptr <RebindableGLFramebufferObject> Ptr;

class BlurBackingFramebuffer :
    public BackingFramebufferInterface
{
    public:

	typedef boost::shared_ptr <BlurBackingFramebuffer> Ptr;

	static Ptr create (const CompSize &sz,
			   GLProgram      *program,
			   GLWindow       *window);
	~BlurBackingFramebuffer ();

	const CompSize & size () const;
	bool drawAtPosition (FramebufferDrawVertexData *,
			     GLMatrix *,
			     const CompRect &windowRelativeRect);

	RebindableBackingFramebufferInterface::Ptr bind (const CompRect &viewport);
	void  unbind (const RebindableBackingFramebufferInterface::Ptr &);

	void setWeights (const std::vector <GLfloat> &weights);

    private:

	BlurBackingFramebuffer (const CompSize &sz,
				GLProgram      *program,
				GLWindow       *window);

	PrivateBackingFramebuffer *priv;
};

class FramebufferPainter
{
    public:

	FramebufferPainter (const BackingFramebufferInterface::Ptr &backing,
			    const CompOutput		  &output,
			    GLfloat			  progress,
			    GLMatrix			  &transform,
			    const CompRect                &geometry,
			    const CompRect                &windowRelativeRect,
			    GLfloat			  alpha,
			    const CompRegion              &clipRegion);
	~FramebufferPainter ();

    private:
	PrivateFramebufferPainter *priv;
};

#endif
