/*
 * Compiz Tiles Plugin
 *
 * fbeffect.h
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _TILES_FBMANAGER_H
#define _TILES_FBMANAGER_H

#include <boost/shared_ptr.hpp>

#include <boost/function.hpp>
#include <GL/gl.h>
#include <core/size.h>
#include <core/rect.h>
#include <core/output.h>

class FramebufferUpdateInterface
{
    public:

    	virtual ~FramebufferUpdateInterface () {}

    	virtual bool fboUpdate () = 0;
};

class FramebufferUpdateQueueInterface
{
    public:

    	virtual ~FramebufferUpdateQueueInterface () {}

    	virtual bool addQueuedUpdate (FramebufferUpdateInterface *) = 0;
	virtual void removeUpdatesFromQueue (FramebufferUpdateInterface *) = 0;
	virtual void dispatch () = 0;
	virtual bool pending () = 0;
};

class FramebufferQueuedUpdateInterface
{
    public:

    	virtual ~FramebufferQueuedUpdateInterface () {}

    	virtual void setFramebufferHolder (FramebufferUpdateQueueInterface *) = 0;
    	virtual void queueUpdate () = 0;
	virtual void dequeueUpdates () = 0;
};

class RebindableBackingFramebufferInterface
{
    public:

	typedef boost::shared_ptr <RebindableBackingFramebufferInterface> Ptr;

	virtual ~RebindableBackingFramebufferInterface () {}

	virtual void rebind () = 0;
};

struct FramebufferDrawVertexData;
class GLMatrix;

class BackingFramebufferInterface
{
    public:

	typedef boost::shared_ptr <BackingFramebufferInterface> Ptr;

	virtual ~BackingFramebufferInterface () {};

	virtual const CompSize & size () const = 0;
	virtual bool drawAtPosition (FramebufferDrawVertexData *,
				     GLMatrix *,
				     const CompRect &windowRelativeRect) = 0;

	virtual RebindableBackingFramebufferInterface::Ptr bind (const CompRect &viewport) = 0;
	virtual void  unbind (const RebindableBackingFramebufferInterface::Ptr &) = 0;

	/* XXX: Move this to BlurInterface or something */

	virtual void setWeights (const std::vector <GLfloat> &weights) = 0;
};

class PrivateUpdatableFramebufferCommunicator;

class UpdatableFramebufferCommunicator :
    public FramebufferQueuedUpdateInterface
{
    public:	

	UpdatableFramebufferCommunicator (FramebufferUpdateQueueInterface *,
					  FramebufferUpdateInterface *);

	void setFramebufferHolder (FramebufferUpdateQueueInterface *);
	void queueUpdate ();
	void dequeueUpdates ();

    private:

	PrivateUpdatableFramebufferCommunicator *priv;
};

class PrivateUpdatableFramebufferForwarder;

class UpdatableFramebufferForwarder :
    public FramebufferUpdateInterface
{
    public:

	typedef boost::function <bool ()> UpdateFunc;

	UpdatableFramebufferForwarder (const UpdateFunc &);

	bool fboUpdate ();

    private:

	PrivateUpdatableFramebufferForwarder *priv;
};

class PrivateFramebufferBinder;

class FramebufferBinder
{
    public:

	FramebufferBinder (const BackingFramebufferInterface::Ptr &backing,
			   const CompOutput &output);
	~FramebufferBinder ();

    private:
	PrivateFramebufferBinder *priv;
};

class PrivateFramebufferUpdateQueue;

class FramebufferUpdateQueue :
    public FramebufferUpdateQueueInterface
{
    public:

	FramebufferUpdateQueue ();

	bool addQueuedUpdate (FramebufferUpdateInterface *);
	void removeUpdatesFromQueue (FramebufferUpdateInterface *);
	void dispatch ();
	bool pending ();

    private:

	PrivateFramebufferUpdateQueue *priv;
};

#endif
