#include <gtest/gtest.h>
#include <gmock/gmock.h>

#include <boost/bind.hpp>
#include <boost/make_shared.hpp>

#include "fbmanager.h"
#include "mockfbmanager.h"

using ::testing::Return;
using ::testing::_;
using ::testing::NiceMock;

class TilesFBTest:
    public ::testing::Test
{
};

TEST(TilesFBTest, TestFramebufferUpdateCallsParent)
{
    MockFramebufferUpdate mfb;
    UpdatableFramebufferForwarder fbf (boost::bind (&FramebufferUpdateInterface::fboUpdate, &mfb));

    EXPECT_CALL (mfb, fboUpdate ());

    fbf.fboUpdate ();
}

TEST(TilesFBTest, TestFramebufferBinderCallsBindUnbindCorrect)
{
    boost::shared_ptr <NiceMock <MockBackingFramebuffer> > mbf = boost::make_shared <NiceMock <MockBackingFramebuffer> > ();

    CompOutput o;

    o.setGeometry (0, 0, 100, 100);

    RebindableBackingFramebufferInterface::Ptr rb;

    EXPECT_CALL (*(mbf.get ()), bind (o)).WillOnce (Return (rb));

    FramebufferBinder (mbf, o);

    // Bug in GTest ?
    //EXPECT_CALL (*mbf, unbind (rb));
}

TEST(TilesFBTest, TestFramebufferUpdateQueueDispatch)
{
    MockFramebufferUpdate mfu;
    FramebufferUpdateQueue uq;

    uq.addQueuedUpdate (&mfu);

    EXPECT_CALL (mfu, fboUpdate ());

    uq.dispatch ();
}

TEST(TilesFBTest, TestFramebufferUpdateQueueDispatchRemove)
{
    MockFramebufferUpdate mfu;
    FramebufferUpdateQueue uq;

    uq.addQueuedUpdate (&mfu);
    uq.removeUpdatesFromQueue (&mfu);

    uq.dispatch ();
}

TEST(TilesFBTest, TestFramebufferUpdateQueuePending)
{
    MockFramebufferUpdate mfu;
    FramebufferUpdateQueue uq;

    uq.addQueuedUpdate (&mfu);

    EXPECT_TRUE (uq.pending ());
}

TEST(TilesFBTest, TestFramebufferUpdateQueueNoPending)
{
    MockFramebufferUpdate mfu;
    FramebufferUpdateQueue uq;

    uq.addQueuedUpdate (&mfu);
    uq.removeUpdatesFromQueue (&mfu);

    EXPECT_FALSE (uq.pending ());
}

TEST(TilesFBTest, TestFramebufferUpdateQueueNoDualDispatch)
{
    MockFramebufferUpdate mfu;
    FramebufferUpdateQueue uq;

    uq.addQueuedUpdate (&mfu);
    uq.addQueuedUpdate (&mfu);

    EXPECT_CALL (mfu, fboUpdate ());

    uq.dispatch ();
}
