/*
 * Compiz Tiles Plugin
 *
 * tiles.cpp
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "geometrysaverestore.h"
#include "resizable.h"
#include "pointerfetch.h"

class PrivateGeometryRelativeSaveRestore
{
    public:

	ResizableInterface *mResizable;
	PointerFetchInterface *mPointerFetch;

	compiz::window::Geometry mSaveGeometry;
	unsigned int mSaveMask;
};

GeometryRelativeSaveRestore::GeometryRelativeSaveRestore (ResizableInterface *resizable,
							  PointerFetchInterface *pointerFetch) :
    priv (new PrivateGeometryRelativeSaveRestore ())
{
    priv->mResizable = resizable;
    priv->mPointerFetch = pointerFetch;
    priv->mSaveMask = 0;
}

compiz::window::Geometry
GeometryRelativeSaveRestore::restoreToRect ()
{
    compiz::window::Geometry r (priv->mSaveGeometry);

    float xScale = priv->mSaveMask & CWWidth ? priv->mSaveGeometry.width () / static_cast <float> (priv->mResizable->windowGeometry ().width ()) : 1.0f;
    float yScale = priv->mSaveMask & CWHeight ? priv->mSaveGeometry.height () / static_cast <float> (priv->mResizable->windowGeometry ().height ()) : 1.0f;

    CompPoint p = priv->mPointerFetch->getPointerPosition ();

    r.setX (p.x () - ((p.x () - priv->mResizable->windowGeometry ().x ()) * xScale));
    r.setY (p.y () - ((p.y () - priv->mResizable->windowGeometry ().y ()) * yScale));
    r.setWidth (priv->mSaveMask & CWWidth ? priv->mSaveGeometry.width () : priv->mResizable->windowGeometry ().width ());
    r.setHeight (priv->mSaveMask & CWHeight ? priv->mSaveGeometry.height () : priv->mResizable->windowGeometry ().height ());
    r.setBorder (0);

    return r;
}

void
GeometryRelativeSaveRestore::saveGeometry (const compiz::window::Geometry &r, unsigned int mask)
{
    priv->mSaveGeometry.applyChange (r, mask);
    priv->mSaveMask |= mask;
}

GeometryRelativeSaveRestore::~GeometryRelativeSaveRestore ()
{
    compiz::window::Geometry r = restoreToRect ();

    priv->mResizable->applyResizeRules (false);
    priv->mResizable->resizeWindow (r);
}
