#include <gtest/gtest.h>
#include <gmock/gmock.h>

#include "geometrysaverestore.h"
#include "mockresizable.h"
#include "mockpointerfetch.h"

using ::testing::Return;
using ::testing::ReturnRef;
using ::testing::_;

class TilesGeometrySaveRestoreTest :
    public ::testing::Test
{
};

TEST(TilesGeometrySaveRestoreTest, TestRestoreWidthOnly)
{
    MockResizable mr;
    MockPointerFetch mpf;

    GeometryRelativeSaveRestore gr (&mr, &mpf);

    gr.saveGeometry (compiz::window::Geometry (0, 0, 200, 200, 0), CWWidth);

    CompPoint p (50, 50);
    CompRect g (20, 20, 100, 100);

    EXPECT_CALL (mr, windowGeometry ()).WillRepeatedly (Return (g));
    EXPECT_CALL (mpf, getPointerPosition ()).WillRepeatedly (ReturnRef (p));

    compiz::window::Geometry r = gr.restoreToRect ();

    EXPECT_EQ (r, compiz::window::Geometry (-10, 20, 200, 100, 0));

    /* Destructor */
    EXPECT_CALL (mr, applyResizeRules (false));
    EXPECT_CALL (mr, resizeWindow (r));
}

TEST(TilesGeometrySaveRestoreTest, TestRestoreHeightOnly)
{
    MockResizable mr;
    MockPointerFetch mpf;

    GeometryRelativeSaveRestore gr (&mr, &mpf);

    gr.saveGeometry (compiz::window::Geometry (0, 0, 200, 200, 0), CWHeight);

    CompPoint p (50, 50);
    CompRect g (20, 20, 100, 100);

    EXPECT_CALL (mr, windowGeometry ()).WillRepeatedly (Return (g));
    EXPECT_CALL (mpf, getPointerPosition ()).WillRepeatedly (ReturnRef (p));

    compiz::window::Geometry r = gr.restoreToRect ();

    EXPECT_EQ (r, compiz::window::Geometry (20, -10, 100, 200, 0));

    /* Destructor */
    EXPECT_CALL (mr, applyResizeRules (false));
    EXPECT_CALL (mr, resizeWindow (r));
}

TEST(TilesGeometrySaveRestoreTest, TestRestoreWidthHeight)
{
    MockResizable mr;
    MockPointerFetch mpf;

    GeometryRelativeSaveRestore gr (&mr, &mpf);

    gr.saveGeometry (compiz::window::Geometry (0, 0, 200, 200, 0), CWWidth | CWHeight);

    CompPoint p (50, 50);
    CompRect g (20, 20, 100, 100);

    EXPECT_CALL (mr, windowGeometry ()).WillRepeatedly (Return (g));
    EXPECT_CALL (mpf, getPointerPosition ()).WillRepeatedly (ReturnRef (p));

    compiz::window::Geometry r = gr.restoreToRect ();

    EXPECT_EQ (r, compiz::window::Geometry (-10, -10, 200, 200, 0));

    /* Destructor */
    EXPECT_CALL (mr, applyResizeRules (false));
    EXPECT_CALL (mr, resizeWindow (r));
}

TEST(TilesGeometrySaveRestoreTest, TestRestoreWidthThenHeight)
{
    MockResizable mr;
    MockPointerFetch mpf;

    GeometryRelativeSaveRestore gr (&mr, &mpf);

    gr.saveGeometry (compiz::window::Geometry (0, 0, 200, 0, 0), CWWidth);
    gr.saveGeometry (compiz::window::Geometry (0, 0, 0, 200, 0), CWHeight);

    CompPoint p (50, 50);
    CompRect g (20, 20, 100, 100);

    EXPECT_CALL (mr, windowGeometry ()).WillRepeatedly (Return (g));
    EXPECT_CALL (mpf, getPointerPosition ()).WillRepeatedly (ReturnRef (p));

    compiz::window::Geometry r = gr.restoreToRect ();

    EXPECT_EQ (r, compiz::window::Geometry (-10, -10, 200, 200, 0));

    /* Destructor */
    EXPECT_CALL (mr, applyResizeRules (false));
    EXPECT_CALL (mr, resizeWindow (r));
}
