/*
 * Compiz Tiles Plugin
 *
 * grabtracker.h
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _TILES_GRABTRACKER_H
#define _TILES_GRABTRACKER_H

#include <boost/function.hpp>
#include <core/point.h>
#include <core/rect.h>

class TileAssignableInterface;
class AnimatableWindowInterface;
class WindowMonitorInterface;
class DrawableWindowInterface;
class DropCommitInterface;
class ResizableInterface;
class MaximizationStateApplicationInterface;
class AnimatableRectInterface;
class DroppableInterface;
class DraggableInterface;
class SnapRemovableInterface;
class ResizeAllocationInterface;
class DragAndDrop;
class DragAndDropInterface;

namespace tile
{
    namespace impl
    {
	typedef boost::function <CompRect ()> GetWorkAreaFunc;
    }
}

class GrabTrackingInterface
{
    public:

	virtual ~GrabTrackingInterface () {}

	virtual bool addGrabbedWindow (TileAssignableInterface *,
				       AnimatableWindowInterface *,
				       WindowMonitorInterface *,
				       DrawableWindowInterface *,
				       DropCommitInterface *,
				       const tile::impl::GetWorkAreaFunc &,
				       const CompPoint &) = 0;
	virtual void removeGrabbedWindow (DropCommitInterface *) = 0;
	virtual void dragGrabbedWindow (ResizableInterface *,
					TileAssignableInterface *,
					WindowMonitorInterface *,
					DrawableWindowInterface *,
					MaximizationStateApplicationInterface *max,
					const boost::shared_ptr <AnimatableRectInterface> &animatable,
					const boost::shared_ptr <DragAndDropInterface> &dnd,
					const boost::shared_ptr <ResizeAllocationInterface> &resizeAllocator,
					const CompPoint &) = 0;

};

class GrabTrackableInterface
{
    public:

	virtual ~GrabTrackableInterface () {}
	virtual void setGrabTracker (GrabTrackingInterface *grabTracker) = 0;
};

class GrabTrackerFinderInterface
{
    public:

	virtual ~GrabTrackerFinderInterface () {}
	virtual GrabTrackingInterface * findTracker (WindowMonitorInterface *wm, const CompPoint &pointer) = 0;
};

#endif
