/*
 * Compiz Tiles Plugin
 *
 * tiles.cpp
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "maximization.h"
#include "resizable.h"
#include "monitor.h"

#define CompWindowStateMaximizedVertMask    (1 <<  2)
#define CompWindowStateMaximizedHorzMask    (1 <<  3)

class PrivateMaximizationStateApplier
{
    public:

	WindowMonitorInterface *mMonitor;
	ResizableInterface *mResizable;
	MaximizableInterface *mMaximizable;
};

MaximizationStateApplier::MaximizationStateApplier (ResizableInterface *resizable,
						    WindowMonitorInterface *monitor,
						    MaximizableInterface *maximizable) :
    priv (new PrivateMaximizationStateApplier)
{
    priv->mMonitor = monitor;
    priv->mResizable = resizable;
    priv->mMaximizable = maximizable;
}

void
MaximizationStateApplier::applyMaximizeState (const CompRect &resizeTarget)
{
    const CompRect &wa = priv->mMonitor->monitorWorkArea ();
    CompRect frameRect = priv->mResizable->toFrameGeometry (resizeTarget);

    unsigned int maximizeMask = 0;

    if (frameRect.width () == wa.width ())
    	maximizeMask |= CompWindowStateMaximizedHorzMask;
    if (frameRect.height () == wa.height ())
    	maximizeMask |= CompWindowStateMaximizedVertMask;

    priv->mMaximizable->maximize (maximizeMask);
}
