#include <gtest/gtest.h>
#include <gmock/gmock.h>

#include "resizable.h"
#include "maximization.h"
#include "mockresizable.h"
#include "mockmaximization.h"
#include "mockmonitor.h"

#define CompWindowStateMaximizedVertMask    (1 <<  2)
#define CompWindowStateMaximizedHorzMask    (1 <<  3)

using ::testing::Return;
using ::testing::_;

class TilesMaximizeApplierTest:
    public ::testing::Test
{
};

TEST(TilesMaximizeApplierTest, TestWidthApplication)
{
    MockWindowMonitor mwm;
    MockResizable     mr;
    MockMaximizable   mm;

    MaximizationStateApplier msa (&mr, &mwm, &mm);

    CompRect monitor (0, 0, 100, 100);
    CompRect target (0, 0, 50, 100);

    EXPECT_CALL (mwm, monitorWorkArea ()).WillOnce (Return (monitor));
    EXPECT_CALL (mr, toFrameGeometry (target)).WillOnce (Return (target));
    EXPECT_CALL (mm, maximize (CompWindowStateMaximizedVertMask));

    msa.applyMaximizeState (target);
}

TEST(TilesMaximizeApplierTest, TestHeightApplication)
{
    MockWindowMonitor mwm;
    MockResizable     mr;
    MockMaximizable   mm;

    MaximizationStateApplier msa (&mr, &mwm, &mm);

    CompRect monitor (0, 0, 100, 100);
    CompRect target (0, 0, 100, 50);

    EXPECT_CALL (mwm, monitorWorkArea ()).WillOnce (Return (monitor));
    EXPECT_CALL (mr, toFrameGeometry (target)).WillOnce (Return (target));
    EXPECT_CALL (mm, maximize (CompWindowStateMaximizedHorzMask));

    msa.applyMaximizeState (target);
}

TEST(TilesMaximizeApplierTest, TestBothApplication)
{
    MockWindowMonitor mwm;
    MockResizable     mr;
    MockMaximizable   mm;

    MaximizationStateApplier msa (&mr, &mwm, &mm);

    CompRect monitor (0, 0, 100, 100);
    CompRect target (0, 0, 100, 100);

    EXPECT_CALL (mwm, monitorWorkArea ()).WillOnce (Return (monitor));
    EXPECT_CALL (mr, toFrameGeometry (target)).WillOnce (Return (target));
    EXPECT_CALL (mm, maximize (CompWindowStateMaximizedVertMask | CompWindowStateMaximizedHorzMask));

    msa.applyMaximizeState (target);
}

TEST(TilesMaximizeApplierTest, TestNoneApplication)
{
    MockWindowMonitor mwm;
    MockResizable     mr;
    MockMaximizable   mm;

    MaximizationStateApplier msa (&mr, &mwm, &mm);

    CompRect monitor (0, 0, 100, 100);
    CompRect target (0, 0, 90, 90);

    EXPECT_CALL (mwm, monitorWorkArea ()).WillOnce (Return (monitor));
    EXPECT_CALL (mr, toFrameGeometry (target)).WillOnce (Return (target));
    EXPECT_CALL (mm, maximize (0));

    msa.applyMaximizeState (target);
}
