/*
 * Compiz Tiles Plugin
 *
 * resizeallocator.h
 *
 * Copyright (c) 2011 Sam Spilsbury <smspillaz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _TILES_RESIZEALLOCATION_H
#define _TILES_RESIZEALLOCATION_H

#include <boost/shared_ptr.hpp>
#include <core/rect.h>
#include "resizable.h"

class ResizeAllocationInterface
{
    public:

	typedef boost::shared_ptr <ResizeAllocationInterface> Ptr;

	virtual ~ResizeAllocationInterface () {}

	virtual const CompRect & allocation () = 0;
	virtual bool setAllocation (const CompRect &) = 0;
	virtual bool tileableFitsAllocation (ResizableInterface *tileable) = 0;
};

class ResizeTargetInterface
{
    public:

	virtual ~ResizeTargetInterface () {}

	virtual const CompRect & target () = 0;
	virtual void setTarget (const CompRect &) = 0;
};

class PrivateResizeAllocator;

class ResizeAllocator :
    public ResizeAllocationInterface
{
    public:

	ResizeAllocator (ResizableInterface *);

	bool setAllocation (const CompRect &);
	const CompRect & allocation ();
	bool tileableFitsAllocation (ResizableInterface *tileable);

    private:

	PrivateResizeAllocator *priv;
};

#endif
